function varargout = ex_1(varargin)
% EX_1    Tool to visualize the 256 celullar automata (CA)
%
%    Author:
%       Thomas Schaffter, PhD student
%       Thomas.Schaffter@epfl.ch
%       http://lis.epfl.ch/161219
%       Laboratory of Intelligent Systems (LIS)
%       Ecole Polytechnique Federale de Lausanne (EPFL)
%       CH-Suisse
%
%    Last modification: 3 nov 2009


% Add the path to the additional m-files
addpath('functions');

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @ex_1_OpeningFcn, ...
                   'gui_OutputFcn',  @ex_1_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

% --- Executes just before ex_1 is made visible.
function ex_1_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to ex_1 (see VARARGIN)

% Choose default command line output for ex_1
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% This sets up the initial plot - only do when we are invisible
% so window can get raised using ex_1.
if strcmp(get(hObject,'Visible'),'off')
    run(handles);
end

% UIWAIT makes ex_1 wait for user response (see UIRESUME)
% uiwait(handles.myApplication);


% --- Outputs from this function are returned to the command line.
function varargout = ex_1_OutputFcn(hObject, eventdata, handles)
% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on button press in randomRule.
function randomRule_Callback(hObject, eventdata, handles)
set(handles.ruleEdit, 'String', randi(255));
run(handles);


% --- Executes during object creation, after setting all properties.
function ruleEdit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function ruleEdit_Callback(hObject, eventdata, handles)
checkParameter(hObject, 0, 255);
run(handles);


% --- Executes during object creation, after setting all properties.
function densityEdit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function densityEdit_Callback(hObject, eventdata, handles)
checkParameter(hObject, 0, 1);
run(handles);


% --- Executes during object creation, after setting all properties.
function widthEdit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function widthEdit_Callback(hObject, eventdata, handles)
checkParameter(hObject, 1,5000);
run(handles);
repaint(hObject, eventdata, handles);


% --- Executes during object creation, after setting all properties.
function timeEdit_CreateFcn(hObject, eventdata, handles)
if ispc && isequal(get(hObject,'BackgroundColor'), get(0,'defaultUicontrolBackgroundColor'))
    set(hObject,'BackgroundColor','white');
end

function timeEdit_Callback(hObject, eventdata, handles)
checkParameter(hObject, 1,5000);
run(handles);
repaint(hObject, eventdata, handles);


% --- Executes when myApplication is resized.
function myApplication_ResizeFcn(hObject, eventdata, handles)
repaint(hObject, eventdata, handles);



% Update the GUI
function repaint(hObject, eventdata, handles)
% get the position of the GUI
ref = get(handles.myApplication, 'Position');
% Update the position of the settings panel
old_units = get(handles.settingsPanel, 'Position');
bkp_height = ref(4)-(old_units(4)+9);
set(handles.settingsPanel, 'Position', [15 bkp_height old_units(3) old_units(4)]);
% Update the position of the rule panel
old_units = get(handles.ruleAxes, 'Position');
bkp_height = bkp_height-old_units(4)-15;
set(handles.ruleAxes, 'Position', [15 bkp_height old_units(3) old_units(4)]);
% Update the position and size of the CA
w = str2double(get(handles.widthEdit, 'String'));
t = str2double(get(handles.timeEdit, 'String'));
new_w = ref(3)-30;
new_h = new_w*t/w;
bkp_height = bkp_height-new_h-15;
set(handles.caAxes, 'Position', [15 bkp_height new_w new_h]);


% Run CA
function run(handles)
% Get settings
ruleCode = str2double(get(handles.ruleEdit, 'String'));
spaceWidth = str2double(get(handles.widthEdit, 'String'));
numTimeSteps = str2double(get(handles.timeEdit, 'String'));
densityRandomFilling = str2double(get(handles.densityEdit, 'String'));
% Compute rule
axes(handles.ruleAxes);
computeTransitionRule(ruleCode);
set(handles.ruleAxes, 'XTickLabel',[]);
set(handles.ruleAxes, 'YTickLabel',[]);
set(handles.ruleAxes, 'XTick',[]);
set(handles.ruleAxes, 'YTick',[]);
axis equal
axis tight
% Compute CA
axes(handles.caAxes);
init = initialState(spaceWidth, densityRandomFilling);
computeCA(ruleCode,init,numTimeSteps);
set(handles.caAxes, 'XTickLabel',[]);
set(handles.caAxes, 'YTickLabel',[]);
set(handles.caAxes, 'XTick',[]);
set(handles.caAxes, 'YTick',[]);
