function [Z img] = computeCA(ruleCode,init,numTimeSteps)
% COMPUTECA    Return the elementary CA as an array (space x time) and as
%              a binary image.
%
%    Author:
%       Thomas Schaffter, PhD student
%       Thomas.Schaffter@epfl.ch
%       http://lis.epfl.ch/161219
%       Laboratory of Intelligent Systems (LIS)
%       Ecole Polytechnique Federale de Lausanne (EPFL)
%       CH-Suisse
%
%    Last modification: 3 nov 2009

spaceWidth = length(init);

% transition rule
rule = computeTransitionRule(ruleCode);

% create and init CA space
Z=ones(numTimeSteps,spaceWidth);
Z(1,:) = init;

% boundary condition
d = zeros(3,spaceWidth);
vec = [4 2 1];
for t=1:numTimeSteps
    y = Z(t,:); % get state at time t
    d(1,:) = [y(spaceWidth) y(1:spaceWidth-1)];
    d(2,:) = y;
    d(3,:) = [y(2:spaceWidth) y(1)];
    newState = rule(8-vec*d); % state at time t+1
    Z(t+1,:) = newState; % save new state
end

% return as image
z = zeros(size(Z));
img = image(cat(3,Z,z,z));


