function [rule img] = computeTransitionRule(ruleCode)
% COMPUTETRANSITIONRULE    Return the binary sequence associated to an
%                          elementary rule code, as well as an image
%                          representing the transition rule.
%
%    Author:
%       Thomas Schaffter, PhD student
%       Thomas.Schaffter@epfl.ch
%       http://lis.epfl.ch/161219
%       Laboratory of Intelligent Systems (LIS)
%       Ecole Polytechnique Federale de Lausanne (EPFL)
%       CH-Suisse
%
%    Last modification: 3 nov 2009

% background color of the rule panel (grey level)
bgColor = 0.6;

% transition rule
rule = bitget(ruleCode,8:-1:1);

% setup an image to display the transition rule
ruleDisplay = bgColor*ones(4,33);
for r=1:8
    a=2+(r-1)*4;
    ruleDisplay(2,a:a+2) = bitget(8-r,3:-1:1);
    ruleDisplay(3,a+1) = rule(r);
end
bg = bgColor*(ruleDisplay==bgColor);
img = image(cat(3,ruleDisplay,bg,bg));